#ifndef COMPOSITE_H
#define COMPOSITE_H

#include "object.h"
#include <vector>
#include <memory>

using namespace std;

class WallCreator;

class Composite : public Object
{
	public:
		~Composite();
		virtual double computeCollisionTime(const Particle &p) const;
		virtual void findCollision(bool delCollision);
		virtual bool testCollision(Particle const &p) const; //tests if this and p shoud collide now
		virtual double mass() const;
		virtual void collisionDirection(const Particle& p,double &dx,double &dy);
		virtual void writeLog(FILE* file);
		virtual void draw(Canvas &) const;
		virtual void run(double dt);
		virtual Rectangle boundingRect() const;
		virtual void getOwners(std::vector<CellId> *v) const; 

		void addObject(Object*);
	private:
		friend class WallCreator;
		class CollisionEvent;
		vector<Object* > objs;
		double m;
		bool ignoreGravity;
		CollisionEvent *collision;
		struct CollisionInfo {
			double time;
			Object *obj;
		};
		CollisionInfo findFirstCollision(const Particle& p) const;
		Composite(Simulation* sim0,double mass,bool ignoreGravity0);  //only WallCreator can create composite
		Composite(Composite &);
protected:
    virtual void doCollisionPrivate(Particle &p);
};

#endif
