#ifndef _HEAP_CPP
#define _HEAP_CPP
#include "myassert.h"
#include <cstdio>
#include <cstdlib>
#include "queueitem.h"
#include <vector>
#ifdef NDEBUG
#define assert2(e,action) ((void)0)
#else
#define assert2(e,action) if (!e) { printf("Assertion failed: "); action; getchar();exit(0);};
#endif



template<class KeyExtractor,class Value>
class Compare {
	public:
		Compare(const KeyExtractor &e):extractor(e) {};
		bool operator()(const Value& x,const Value& y) {
			return extractor.getKey(&x)<extractor.getKey(&y);
		}
	private:
		KeyExtractor extractor;
};

/*
** Halda, ktora uchovava na vrchu najmensi prvok podla Compare
** KeyCompare(a,b) == a<b
** bool KeyCompare(const Value&,const Value&);
** Prvky su typu Value, co musi byt potomok QueueItem
** Odkazy na prvky su uchovavane pomocou typu Pointer
*/

template <class Value,typename Compare,class Pointer=Value*>
class Heap {
	private:
		std::vector<Pointer> storage;
		Compare compare;
		int parent(int x) {return (x+1)/2-1;};
		int leftSon(int x) {return (x+1)*2-1;};
		int rightSon(int x) {return (x+1)*2;};
		void swapAt(int x,int y) {
			Pointer tmp=storage[x];
			storage[x]=storage[y];
			storage[y]=tmp;
			storage[x]->QueueItem::position=x;
			storage[y]->QueueItem::position=y;
		};
		void shiftDown(int index) { 
			bool done=false;
			do {
				int ls=leftSon(index);
				if (ls>=storage.size()) done=true;                   //vrchol nema synov
				else {
					Pointer min=storage[index];
					int minindex=index;
					if (compare(*storage[ls],*min)) {
						min=storage[ls];
						minindex=ls;
					};
//urcite existuje lavy syn vrcholu index, ale existuje pravy syn?
					if (ls+1<storage.size())
						if (compare(*storage[ls+1],*min)) {
						min=storage[ls+1];
						minindex=ls+1;
						};
						if (minindex==index)
							done=true;
						else {
							swapAt(index,minindex);
							index=minindex;
						};
				};
			} while (!done);
		};    
		void shiftUp(int index) {
			while (index>0 && compare(*storage[index],*storage[parent(index)])) {
				swapAt(index,parent(index));
				index=parent(index);
			};
		};
		void removeAt(int index) {
			int size=storage.size()-1;
			swapAt(index,size);
			
			storage[index]->QueueItem::position=index;
			storage[size]->QueueItem::position=-1;
			storage.pop_back();
			if (index<size) {  // if index==size, we do not need to do anything ( and we can't access storage[size])
				shiftUp(index);
				shiftDown(index);
			};
			
		};
		bool test() {
			bool result=true;
			for (int i=0;i<storage.size();i++) {
				int l=leftSon(i);
				int r=rightSon(i);
				if (l<storage.size()) result&=!compare(*storage[l],*storage[i]);
				if (r<storage.size()) result&=!compare(*storage[r],*storage[i]);
/*            assert2(result,
				printf("First pass: at pos %d\n"
				"storage[i] = %d\n"
				"storage[l] = %d\n"
				"storage[r] = %d\n"
				i,storage[i]->x,storage[l]->x,storage[r]->x));/**/
				result&= (storage[i]->QueueItem::position == i) ||
						(storage[i]->QueueItem::position == -1) ;
				assert(result);
			};
			return result;
		};    
	public:
		Heap(int n,Compare _comp=Compare()) :compare(_comp) {
			storage.reserve(n);
			test();
		};
		void insert(Pointer value) {
			storage.push_back(value);
			int index=storage.size()-1;
			value->QueueItem::position=index;
			shiftUp(index);
		};
		void remove(Pointer value) {
			int pos=value->QueueItem::position;
			if (pos>=0)  //ak pos==-1, prvok nie je v halde => nerob nic
				removeAt(pos);
		};
		Pointer getMin() {return storage[0];}
		void popMin() {
			if (storage.size()>0)
				removeAt(0); 
		}
		void reset(Pointer value) {         //ak value uz je v halde,umiestni ho na spravne miesto
			int pos=value->QueueItem::position;
			if (pos==-1) return;
			shiftUp(pos);
			shiftDown(pos);
		};
};
#endif //_HEAP_CPP
