#include <cassert>
#include <cstdio>
#include <cstdlib>

#ifdef NDEBUG
#define assert2(e,action) ((void)0)
#else
#define assert2(e,action) if (!e) { printf("Assertion failed: "); action; exit(0);};
#endif

template <class Value,typename Compare>
class Heap;    

class HeapItem {
template <class Value,typename Compare>    
friend class Heap;
private:
    int position;
};


/*
** Halda, ktora uchovava na vrchu najmensi prvok podla KeyCompare
** KeyCompare(a,b) == a<b
** bool KeyCompare(const Value&,const Value&);
** Prvky su typu Value, co musi byt potomok HeapItem
*/
template <class Value,typename Compare>
class Heap {
private:
    Value **storage;
    Compare compare;
    int size,maxSize;
    int parent(int x) {return (x+1)/2-1;};
    int leftSon(int x) {return (x+1)*2-1;};
    int rightSon(int x) {return (x+1)*2;};
    void swapAt(int x,int y) {
        Value *tmp=storage[x];
        storage[x]=storage[y];
        storage[y]=tmp;
        storage[x]->HeapItem::position=x;
        storage[y]->HeapItem::position=y;        
    };    
    
    void removeAt(int index) {
        --size;
        storage[index]=storage[size]; storage[index]->HeapItem::position=index;
        bool done=false;
        do {
            int ls=leftSon(index);
            if (ls>=size) done=true;                   //vrchol nema synov
            else {
                Value* min=storage[index];
                int minindex=index;
                if (compare(*storage[ls],*min)) {
                    min=storage[ls];
                    minindex=ls;
                };    
//urcite existuje lavy syn vrcholu index, ale existuje pravy syn?
                if (ls+1<size)
                    if (compare(*storage[ls+1],*min)) {
                        min=storage[ls+1];              
                        minindex=ls+1;
                    };      
                if (minindex==index)
                    done=true;
                else {
                    swapAt(index,minindex);
                    index=minindex;
                };    
            };
        } while (!done);
    };    
public:
    Heap(int n) : size(0),maxSize(n),storage(new Value*[n]) {};
    void insert(Value *value) {
        //assert(size<maxSize-1);
        int index=size;
        storage[size++]=value;
        while (index>0 && compare(*storage[index],*storage[parent(index)])) {
            swapAt(index,parent(index));
            index=parent(index);
        };    
        value->HeapItem::position=index;
    };    
    void remove(Value *value) {
        removeAt(value->HeapItem::position);
    };    
    Value *getMin() {return storage[0];}
    void popMin() {
        removeAt(0); 
    }
    bool test() {
        bool result=true;
        for (int i=0;i!=size;i++) {
            int l=leftSon(i);
            int r=rightSon(i);
            if (l<size) result&=!compare(*storage[l],*storage[i]);
            if (r<size) result&=!compare(*storage[r],*storage[i]);
            assert2(result,
                    printf("First pass: at pos %d\n"
                    "storage[i] = %d\n"
                    "storage[l] = %d\n"
                    "storage[r] = %d\n"
                    ,i,storage[i]->x,storage[l]->x,storage[r]->x));
            result&= storage[i]->HeapItem::position == i;
            assert2(result,printf("2nd: at pos %d",i));
        };    
    };    
};


class Int : public HeapItem {
    public:
        int x;
        Int() : x(0) {};
        Int(int x0):x(x0) {};
};

struct comp {
bool operator()(const Int &a,const Int &b){return a.x<b.x;};
};

int test() {
    Heap<Int,comp> A(100);
    Int B[100];
    for (int i=0; i!=100;i++) {
        printf("i=%d\n",i);
        B[i].x=rand()%1000;
        A.insert(&B[i]);
//        A.test();
    };   
    for (int i=0;i!=10000;i++) {
        Int *c;
        if (rand()%2>=0) 
            c=A.getMin();
        else
            c=&B[rand()%100];
        printf("%3d.  %d\n",i,c->x);
//        A.test();        
        A.remove(c);
//        A.test();
        c->x=rand()%1000;
        A.insert(c);
//        A.test();
    };  
/*    printf("All right\n");
    getchar();getchar();
    getchar();
  */  
};
