#ifndef OBJECT_H
#define OBJECT_H
//#include "simulation.h"

#include <cstdio>
#include "common.h"
#include <vector>
#include "counted.h"

class Particle;
class Simulation;
class Canvas;
class Composite;

struct Rectangle {
	double x1,y1,x2,y2;
	Rectangle(double x1_,double y1_,double x2_,double y2_):x1(x1_),y1(y1_),x2(x2_),y2(y2_) {};
};

class Object : public Counted<Object>
{
	friend class Composite;
	private:
#		ifndef NDEBUG
		double lastCollision;
#		endif
	protected:
		double vx;
		double vy;
		Simulation* sim;
		virtual void doCollisionPrivate(Particle &p);
	public:
		Object(Simulation *sim0): vx(0),vy(0),sim(sim0) {};
		virtual ~Object() {};
		virtual double computeCollisionTime(const Particle &p) const=0;
		// add new collision event into simulation
		virtual void findCollision(bool delCollision)=0;
		virtual void doCollisionWith(Particle& p);
		virtual bool testCollision(Particle const &p) const=0; //tests if this and p shoud collide now
		virtual double mass() const=0;
		virtual void collisionDirection(const Particle& p,double &dx,double &dy)=0;
		virtual void run(double dt)=0;
		
		virtual void writeLog(FILE* file)=0;	
		virtual CellId getOwner() const {return -1;};
		// inserts owning cells of this object to v (with v->push_back())
		// assumes that v is an initialized vector
		// Includes neighbouring cells (ie. returnes all cells whose 
		// particles can collide with this object)
		virtual void getOwners(std::vector<CellId>* v) const=0;
		
		virtual void draw(Canvas&) const=0;
		virtual Rectangle boundingRect() const=0;
		

		void setv(double vx,double vy);
		double getvx() {return vx;}
		double getvy() {return vy;}
		
		void setLastCollision(double t) {
#			ifndef NDEBUG
			lastCollision=t;
#			endif
		};
		double getLastCollision() {
#			ifndef NDEBUG
			return lastCollision;
#			else
			return -1.0;
#			endif
		};
};

#endif // OBJECT_H

