#include "simulation.h"
#include "collisionEvent.h"
#include <cmath>
#include "common.h"
#include "gui/canvas.h"
#include <vector>

//Particle *Particle::invalidParticle=new Particle(0,0,0,0,0,0);

Particle::compare::compare(Simulation *sim0):sim(sim0) {};

bool Particle::compare::operator()(const Particle& a, const Particle& b) {
	assert(false);
    //not used anymore..
/*    double ta=a.collisionTime+sim->getCell(a.owner).getClock();
	double tb=b.collisionTime+sim->getCell(b.owner).getClock();
	return ta<tb;*/
};

void Particle::considerCollisionWith(Object &o) {
	double dt=o.computeCollisionTime(*this);
	double t=dt + sim->getCell(owner).getClock();
	// dt is relative to now, t is absolute
	if (t<collision->getTime()) {
		if (dt<-eps)
			dt=o.computeCollisionTime(*this);
		assert(dt>=-eps);
		
		sim->removeEvent(collision);
		collision=new CollisionEvent(this,&o,t);
		sim->addEvent(collision);
	} else {
		if (&o==collision->getCollisioner())
			collision->invalidate();
	}
}; 

struct CollisionInfo {
	double collisionTime;
	Object *collisioner;
	Particle *p;
	double gravity;
	bool considerCollision(Object *o) {
		double pom=collisionTime;
		double t=o->computeCollisionTime(*p);
		if (t<collisionTime) {
			if (t<-eps)
				t=o->computeCollisionTime(*p);
			assert(t>=-eps);
			collisionTime=t;
			collisioner=o;
		};
	}; 
};

void Particle::findCollision(bool delCollision) {

	CollisionInfo colinfo;
	colinfo.collisionTime=infinity;
	colinfo.p=this;
	colinfo.gravity=sim->getGravity();
	int wallCount=sim->getWallCount();
	int neighbourCount=sim->getNeighbourCount(owner);
	for (int j=0;j!=neighbourCount;j++) {
		CellId ci=sim->getNeighbour(owner,j);
		sim->synchronize(ci);
	};
	for (int j=0;j!=neighbourCount;j++) {
		CellId ci=sim->getNeighbour(owner,j);
		//sim->invalidateCollisionersOf(*this,ci);
		Cell& cell=sim->getCell(ci);

		for (List<Particle>::iterator i=cell.getParticleIter();
				   i.valid();++i) {
					   Particle &p=*i;
					   if (&p==this) continue;
					   colinfo.considerCollision(&p);
				   };
	};
	for (int i=0;i!=wallCount;i++) {
		Wall &w=sim->getWall(i);
		colinfo.considerCollision(&w);
	};
	int compositeCount=sim->getCompositeCount();
	for (int i=0;i!=compositeCount;i++) {
		Object &o=sim->getComposite(i);
		colinfo.considerCollision(&o);
	};	
	int borderCount=sim->getBorderCount(owner);
	for (int i=0;i!=borderCount;i++) {
		CellBorder *cb=sim->getBorder(owner,i);
		if (cb) colinfo.considerCollision(cb);
	};
	
	if (delCollision) {
		sim->removeEvent(collision);
	};
	collision=new CollisionEvent(this,colinfo.collisioner,
								 colinfo.collisionTime+sim->getCell(owner).getClock());
	sim->addEvent(collision);
};

Particle::~Particle()
{

#ifndef NDEBUG
	// we mark the memory that is going to be free
	x=infinity;
	y=infinity;
	m=infinity;
	r=infinity;
#endif

}


//void Particle::invalidateCollisioner(Object &o) 

bool Particle::doCollision() {
	if (!equal(collision->getCollisioner()->computeCollisionTime(*this),0)) {
		double tc=collision->getCollisioner()->computeCollisionTime(*this);
		assert(false);
	};
	collision->getCollisioner()->doCollisionWith(*this);
	return true;
};    


void Particle::writeLog(FILE* logFile) {
//	if (getId()!=187 && getId()!=190) return;
	double ownerClock=-infinity;
	if (sim->valid(owner))
		ownerClock=sim->getCell(owner).getClock();
	double collTime=-infinity;
	Object* collisioner=0;
	if (collision) {
		collTime=collision->getTime();
		collisioner=collision->getCollisioner();
	};
	int collisionerId=collisioner? collisioner->getId() : -1;
	fprintf(logFile,"id:%2d, pos: %lf,%lf, v: %lf,%lf, crash with %d at %lf,owner: %d at %lf,last:%lf\n",
	getId(),x,y,vx,vy,collisionerId,collTime,owner,ownerClock,getLastCollision());

};    

void Particle::test() {
	assert(false);
/*    Object *oldCollisioner=collisioner;
	double oldCollisionTime=collisionTime;
	findCollision(sim,*this);
	bool a=collisionTime+eps>oldCollisionTime;
	assert(a);*/
};    
/*
   dx,dy,vx,vy
   |(dx+t*vx,dy+t*vy)|=2*r
   |(dx+t*vx,dy+t*vy)|^2=4*r^2
   (dx+t*vx)^2 + (dy+t*vy)^2 = 4*r^2
   dx^2 + 2*t*vx*dx + t^2*vx^2
   t^2*(vx^2+vy^2) + 2*t*(vx*dx+vy*dy) + (dx^2+dy^2-4*r^2) = 0
   */
//vracia cas zrazky this a p, vysledok infinity znamena, ze sa nezrazia

//double computeCollisionTime(const Particle &p1,const Particle &p2) {
double Particle::computeCollisionTime(const Particle& p) const {
	const Particle &p1=*this;
	const Particle &p2=p;
	double dx=p1.x-p2.x;
	double dy=p1.y-p2.y;
	double vx=p1.vx-p2.vx;
	double vy=p1.vy-p2.vy;
//    if (((vx>0) == (dx>0)) && ((vy>0) == (dy>0)))  // this implies B>=0
//        return infinity;
	double B=2*(vx*dx+vy*dy);
	if (B>=0) return infinity;    //B determines, if the particles are closing (B<0) or not  (B>0)
    
	double r=p1.radius()+p2.radius();
	double A=sqr(vx)+sqr(vy);
	if (equal(A,0)) return infinity; //A=0 => vx=vy=0 => they won't collide
	double C=sqr(dx)+sqr(dy)-sqr(r);
	double D=sqr(B)-4*A*C;
/*
Can we do it faster? We need to compute D, that is:
	4*(vx*dx+vy*dy)^2 - 4*(vx^2+vy^2)*(dx^2+dy^2-r^2)
	4*(vx^2*dx^2 + 2*vx*dx*vy*dy + vy^2*dy^2 - vx^2*dx^2 - vx^2*dy^2 - vx^2*r^2 - vy^2*dx^2 - vy^2*dy^2 - vy^2*r^2)
	4*(2*vx*dx*vy*dy - vx^2*dy^2 - vx^2*r^2 - vy^2*dx^2 - vy^2*r^2)
	4*((vx*dy - vy*dx)^2 - r^2*A)
	C is only needed to compute D, thus we can delete that.    
	The original way costs 5 multiplications, 3 additions.
	The new way costs 5
*/
	if (D>=0) {
		return (-B-sqrt(D))/2/A;   //we already know that A!=0
	//        double pom=hypot(dx+t*vx,dy+t*vy);
	//        assert(equal(pom,2*Particle::radius));
	} 
	else return infinity;
}
/*
   |px+pv*t-w.pos|= r
   (px-w.pos+-r) + pv*t=0
   t=(w.pos-px+-r)/pv
   */
/*
   |px + pv*t - g*t*t/2| = r
   (px-w.pos) + pv*t - g/2*t^2  = +-r
   -g/2*t^2 + pv*t + (px-w.pos+-r) = 0 


*/
/*
		void doCollision(Particle &p1,Particle& p2) {
// vektory p1.v, p2.v vyjadrime v suradnicovej sustave danej vektormi a,b
// kde a ma smer (dx,dy), b je kolme na a
// a, b su normovane
		double ax=p1.x-p2.x; 
		double ay=p1.y-p2.y;
		double a=hypot(ax,ay);
		assert(equal(a,2*Particle::radius));
		ax/=a; ay/=a; 
		double bx=ay;  
		double by=-ax;    // b je kolme na a
		double v1a=p1.vx*ax + p1.vy*ay;
		double v1b=p1.vx*bx + p1.vy*by;    
		double v2a=p2.vx*ax + p2.vy*ay;    
		double v2b=p2.vx*bx + p2.vy*by;
//zrazka sposobi, ze sa vymenia zlozky v1a a v2a
		double pom=v1a; v1a=v2a; v2a=pom;
		p1.vx=v1a*ax+v1b*bx;
		p1.vy=v1a*ay+v1b*by;
		p2.vx=v2a*ax+v2b*bx;
		p2.vy=v2a*ay+v2b*by;            
};

		void doCollision(Particle &p,Wall& w) {
		if (w.type==wVertical)
		p.vx=-p.vx;
		else
		p.vy=-p.vy;
};  

		void doCollision(Object &o1,Object &o2) {

};
*/
void Particle::updateOwner() {
	if (owner!=invalidOwner)
		sim->getCell(owner).remove(this);
	owner=sim->findOwnerOf(*this);
	sim->synchronize(owner);
	sim->getCell(owner).insert(this);
};

void Particle::error() {
	int x=5;
};

void Particle::collisionDirection(const Particle& p,double &dx,double &dy) {
	dx=x-p.x;
	dy=y-p.y;
};

bool Particle::testCollision(const Particle &p) const {
	const ParticleInfo p1(p);
	
	double d=hypot(p1.x()-x,p1.y()-y);
	return equal(d,radius()+p1.radius());
};

void Particle::draw(Canvas &c) const {
	c.drawCircle(int(x),int(y),int(r));
	//c.drawLine(int(x),int(y),int(x+vx*2*r),int(y+vy*2*r));
}

Rectangle Particle::boundingRect() const {
	return Rectangle(x-r,y-r,x+r,y+r);
};


void Particle::getOwners(std::vector<CellId> *v) const {
	CellId owner1=sim->findOwnerOf(*this);
	assert(owner1>=0);
	for (int i=0;i<sim->getNeighbourCount(owner1);i++) {
		CellId c=sim->getNeighbour(owner1,i);
		v->push_back(c);
	};
};

void Particle::run( double dt ){
	double gravity=sim->getGravity();
	x+=vx*dt;
	y+=vy*dt - gravity*dt*dt/2;
	vy-=gravity*dt;
	if (r>0)  {// if r==0, particle probably belong to some Composite object, and they are allowed to exit the area
		assert(x>=0 && y>=0 ); 
		assert(x<=sim->getMaxX());
		assert(y<=sim->getMaxY());
	};
};

Particle::Particle( Simulation * sim, double x0, double y0, double vx0, double vy0, double r0, double m0 ):Object(sim),x(x0),y(y0),
		m(m0),r(r0),owner(invalidOwner),collision(0)
{
	vx=vx0;vy=vy0;
};

