#ifndef PARTICLE_H
#define PARTICLE_H 
#include "wall.h"
#include "object.h"
#include "common.h"
//#include "simulation.h"
#include "list.h"
#include "queueitem.h"
#include "collisionEvent.h"
#include <boost/intrusive_ptr.hpp>

class Simulation;

class ParticleInfo;
template<class>
class ParticleProxyBase;
class ParticleProxy;
class ParticleKeyExtractor;
class Wall;

double computeCollisionTime(const Particle&, const Particle&);
double computeCollisionTime(const Particle&,const Wall&);

class Particle: public Object,public ListItem<Particle>
{
	template<class>
	friend class ParticleProxyBase;

	friend class ParticleInfo;
	friend class ParticleProxy;
	friend void Object::doCollisionWith(Particle &);
	friend class Wall; // !!! this is only a temporary solution and should be deleted
	friend class ParticleKeyExtractor;
	private:
		static const CellId invalidOwner=-1;
		double x,y,m,r;
		boost::intrusive_ptr<CollisionEvent> collision;
		CellId owner;
	public:
		static Particle *invalidParticle; 
		Particle(Simulation* sim,double x0,double y0, double vx0,double vy0,double r0,double m0);
		~Particle();
		double computeCollisionTime(const Particle &p) const ;
//        { return ::computeCollisionTime(p,*this);};
		bool doCollision(); 
		void findCollision(bool delCollision);
		void updateOwner();
//		double getCollisionTime() {return collisionTime;};
//		Object *getCollisioner() {return collisioner;};
		void run(double dt);
		void error();
		void invalidateCollisioner(Object& o) {
			if (collision)
				if (collision->getCollisioner()==&o) {
				collision->invalidate();
				}

		}; 
		void writeLog(FILE* file);
		void test();
		friend double computeCollisionTime(const Particle&, const Particle&);
		friend double computeCollisionTime(const Particle&,const Wall&);

		struct compare;
		friend struct compare;
		struct compare {
			private:
				Simulation *sim;
			public:
				compare(Simulation *sim0);
				bool operator()(const Particle& a, const Particle& b);
		};
			void invalidateOwner();
		virtual CellId getOwner() const {return owner;};
		virtual void getOwners(std::vector<CellId> *v) const; 
		double mass() const {return m;}
		virtual void collisionDirection(const Particle& p,double &dx,double &dy);
		double radius() const {return r;}
		virtual bool testCollision(const Particle &p) const;
		
		virtual void draw(Canvas &) const;
		virtual Rectangle boundingRect() const;
		
		void considerCollisionWith(Object& o);
		
};

/* To avoid typing the same twice, we declare ParticleProxyBase.
 * Its template argument is either Particle or const Particle */
template<class Particle_>
class ParticleProxyBase {
	protected:
		Particle_ &p;
	public:
		ParticleProxyBase(Particle_  &p0):p(p0) {};
		double x() const {return p.x;}
		double y() const {return p.y;}
		double vx() const {return p.vx;}
		double vy() const {return p.vy;}
		double radius() const {return p.radius();}
		int id() const {return p.getId(); }
		void draw(Canvas& c) const { p.draw(c);}

};

class ParticleInfo:public ParticleProxyBase<const Particle>  {
	public:
		ParticleInfo(Particle const &p0):ParticleProxyBase<const Particle>(p0) {};
};

class ParticleProxy:public ParticleProxyBase<Particle> {
	public:
		ParticleProxy(Particle &p0):ParticleProxyBase<Particle>(p0) {};
		void setv(double vx,double vy) {p.setv(vx,vy);}
};

/*class ParticleInfo { 
	Particle const &p;
	public:
		ParticleInfo(Particle  const &p0):p(p0) {};

		double x() const {return p.x;}
		double y() const {return p.y;}
		double vx() const {return p.vx;}
		double vy() const {return p.vy;}
		double radius() const {return p.radius();}
		int id() const {return p.getId(); }
		void draw(Canvas& c) const { p.draw(c);}
	//	void setv(double vx,double vy) {p.setv(vx,vy);}
};
*/
#endif // PARTICLE_H

