#ifndef WALL_H
#define WALL_H
//#include "particle.h"
#include "object.h"
class Particle;
class Wall;
class Canvas;

double computeCollisionTime(const Particle &p,const Wall &w);
void doCollision(Particle &p,Wall& w);
enum WallType {wHorizontal,wVertical};
class WallInfo;
class Wall : public Object {
	friend class WallInfo;
	friend double computeCollisionTime(const Particle&,const Wall&);
	friend void doCollision(Particle&,Wall&);
	private:
		double pos; 
		double m;
		double start,end;
		WallType type;
		bool ignoreGravity;
		double temperature;
		double heatTransfer;
	protected:
		virtual void doCollisionPrivate(Particle &p);
	public:
		Wall(Simulation *sim0,WallType type0, double pos0,double start0,double end0,double m0,bool ignoreGravity=false);
		~Wall();
//		void set(WallType type,double pos) {this->type=type; this->pos=pos;};
		virtual double computeCollisionTime(const Particle &p) const {
			return ::computeCollisionTime(p,*this);
		};
	// Wall is a passive object, it does not look for collisions 
		virtual void findCollision(bool) {};
		virtual void collisionDirection(const Particle& p,double &dx,double &dy);
		virtual bool testCollision(Particle const &p) const;
		virtual double mass() const {return m;}
		void writeLog(FILE* file);
		virtual void draw(Canvas &) const;
		virtual void run(double dt);
		double getPos() {return pos;}
		virtual Rectangle boundingRect() const;
		virtual void getOwners(std::vector<CellId> *v) const; 
		void setTemperature(double temp);
		// heatTransfer decides how effective the heat transfer is in collisions
		// 0 means no tranfer (temperatures do not change in a collision, // 1 means complete transfer (temperatures are equal after a collision)
		void setHeatTransfer(double ht);
};

class WallInfo {
	Wall& w;
	public:
		WallInfo(Wall &w0):w(w0) {};
		WallType type() {return w.type;}
		double pos() {return w.pos;}
		double start() {return w.start;}
		double end() {return w.end;}
		void draw(Canvas &c) {w.draw(c);}
};

#endif // WALL_H

