#include "wallCreator.h"
#include "composite.h"
#include "wall.h"
#include "simulation.h"

WallCreator::WallCreator(Simulation *sim0,double x0,double y0,double mass):sim(sim0),xend(x0),yend(y0),obj(new Composite(sim0,mass,false)),
lastDir(wHorizontal),firstSegment(true),ignoreGravity(false),m(mass) {

};

WallCreator::WallCreator(Simulation *sim0,double x0,double y0,double mass,bool ignoreGravity0):sim(sim0),xend(x0),yend(y0),obj(new Composite(sim0,mass,ignoreGravity0)),
lastDir(wHorizontal),firstSegment(true),ignoreGravity(ignoreGravity0),m(mass) {

};

WallCreator::~ WallCreator() {
	if (obj) {
		addParticle();
		sim->addObject(obj);
		obj=0;
	};
};

void WallCreator::addSegmentRel(WallType dir,double len,double ht,double temp) {
	if (dir==wHorizontal)
		addSegmentAbs(dir,xend+len,ht,temp);
	else
		addSegmentAbs(dir,yend+len,ht,temp);
	firstSegment=false;
};

void WallCreator::addSegmentAbs(WallType dir,double end,double ht,double temp) {
	/* We only add particles at "corners", not at straight parts of wall/
	   This is not only optimalization; it is also vital for 
	   StaticWallCreator, because static walls cannot contain particles
	   (but can be composed of more parts, which is the point of their
	   existence)
	*/
	if (firstSegment || !(lastDir==dir))
		addParticle();
	if (dir==wHorizontal) {
		Wall *w=new Wall(sim,dir,yend,xend,end,m,ignoreGravity);
		w->setHeatTransfer(ht);
		w->setTemperature(temp);
		obj->addObject(w);
		xend=end;
	} else {
		Wall *w=new Wall(sim,dir,xend,yend,end,m,ignoreGravity);
		w->setHeatTransfer(ht);
		w->setTemperature(temp);
		obj->addObject(w);
		yend=end;
	};
	firstSegment=false;
	lastDir=dir;
};

void WallCreator::addParticle() {
	if (xend>-infinity && xend<infinity && yend>-infinity && yend<infinity) {
		assert(!ignoreGravity);
		obj->addObject(new Particle(sim,xend,yend,0,0,0,m));
	};
}


StaticWallCreator::StaticWallCreator( Simulation * sim, double pos, double  mass, WallType dir0):
		wc(sim,
		   dir0==wHorizontal ? -infinity : pos,
		   dir0==wHorizontal ? pos : -infinity,
		   mass,
		   true
		),
		dir(dir0)
{
	wc.ignoreGravity=true;
}

void StaticWallCreator::addSegment( double end,double ht,double temp )
{
	wc.addSegmentAbs(dir,end,ht,temp);
}

/*
Composite *StaticWallCreator::get() {
	if ((dir==wHorizontal && wc.xend < infinity)
			|| (dir==wVertical && wc.yend < infinity))
		wc.addSegmentAbs(dir,infinity);
	return wc.get();
};
*/

StaticWallCreator::~StaticWallCreator() {
	if ((dir==wHorizontal && wc.xend < infinity)
			|| (dir==wVertical && wc.yend < infinity))
		wc.addSegmentAbs(dir,infinity);
// now wc's destructor gets called
};
