#include "wall.h"
#include "particle.h"
#include "composite.h"
#include <vector>

class Composite;
class Simulation;
class StaticWallCreator;

/* WallCreator is the only way to create walls 
 * It adds the resulting wall into Simulation automatically upon its destruction */
class WallCreator {
	friend class StaticWallCreator;
	public:
		WallCreator(Simulation *sim,double x0,double y0,double mass);
		//adds segment beginning at current position, with length len
		void addSegmentRel(WallType dir,double len,double heatTransfer=0,double temperature=0);
		//adds segment ending at end
		void addSegmentAbs(WallType dir,double end,double heatTransfer=0,double temperature=0);
		~WallCreator();
	private:
		WallCreator(Simulation *sim,double x0,double y0,double mass,bool ignoreGravity);
    void addParticle();
		double xend,yend,m;
		Composite *obj;
		Simulation *sim;
		WallType lastDir;
		bool firstSegment;
		bool ignoreGravity;
};

/* StaticWallCreator is used to create static walls, ie. those that does
   not undergo gravitation. But there is a restriction - static wall must
   be straight and infinite. StaticWallCreator enforces this restrictrion by its
   interface.
*/
class StaticWallCreator {
	public:
		StaticWallCreator(Simulation *sim,double pos,double mass, WallType dir);
		void addSegment(double end,double ht=0,double temp=0);
		~StaticWallCreator();
	private:
		WallCreator wc;
		WallType dir;
};