#define QT_THREAD_SUPPORT
#include <qpainter.h>
#include <qinputdialog.h>
#include <qapplication.h>
#include <cstdlib>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qfiledialog.h>
#include <qthread.h>
#include <qmessagebox.h>
#include "gas/simulation.h"
#include <cmath>
#include <stdexcept>
#include <qtimer.h>
#include <qscrollview.h>
#include <qdatetime.h>
#include "gui/form1.moc.cpp"
#include "scripting/init_sim.h"
#include "gui/displaySettings.h"
#include <gas/safevector.h>
#include "gui/qtCanvas.h"

class DrawingThread;

QButtonGroup *g;
Simulation *sim;
QLabel *pixmapLabel1;
QScrollView *scrollView;
DrawingThread *drawingThread;
QMutex running_mutex;

QString scriptName = "scripts/default.py";

int bucketWidth = 30, bucketHeight = 30;
double speedInterval = 0.1;

const double maxv = 1;

class DrawingThread: public QThread {
		Form1 *form;
		QWaitCondition should_draw;
		bool should_quit;
	public:
		DrawingThread( Form1* f ) : form( f ), should_quit( false ) {}
		;
		void run() {}
		;
		void do_draw() {
			should_draw.wakeOne();
		};
		void quit() {
			should_quit = true;
		};
};

class MouseEventFilter: public QObject {
	private:
		Form1* form;
	protected:
		bool eventFilter(QObject *o, QEvent *e) {
			if (e->type()==QEvent::MouseButtonPress) {
				if (form->checkBox1->isChecked())  // proceed only if sim is not running
					return true;
				QMouseEvent *m=(QMouseEvent*)e;
				SimulationInfo si(sim);
				for (int i=0;i<sim->getParticleCount();i++) {
					ParticleInfo p=si.getParticle(i);	
					if ((hypot(p.x()-m->x(),p.y()-m->y()))<p.radius())
						qDebug("Particle #%d",p.id());
				};
				return true;
			} else {
				return false;
			};
		};
	public:
		MouseEventFilter(Form1* f):form(f) {};
};

void Form1::init() {
	scrollView = new QScrollView( this );
	pixmapLabel1 = new QLabel( scrollView->viewport() );
	pixmapLabel1->setGeometry( 0, 0, 100, 100 );
	QPixmap *pixmap = new QPixmap( pixmapLabel1->size() );
	pixmap->fill();

	scrollView->setGeometry( 0, 70, width(), height()-90 );
	//	QLabel *l=new QLabel(sv->viewport());
	scrollView->enableClipper( true );
	scrollView->addChild( pixmapLabel1 );
	//scrollView->setSizeHint(QSize(1000,1000));
	//	sv->resizeContents(1000,1000);

	pixmapLabel1->setPixmap( *pixmap );

	g = new QButtonGroup( this, "ViewGroupBox" );


	QRadioButton *c1;
	c1 = new QRadioButton( 0 );
	c1->setText( "Particles" );
	c1->setChecked( true );
	View->insertItem( c1 );
	g->insert( c1 );

	c1 = new QRadioButton( 0 );
	c1->setText( "Density" );
	View->insertItem( c1 );
	g->insert( c1 );

	
	c1 = new QRadioButton( 0 );
	c1->setText( "Temperature" );
	View->insertItem( c1 );
	g->insert( c1 );

	c1 = new QRadioButton( 0 );
	c1->setText( "Speed" );
	View->insertItem( c1 );
	g->insert( c1 );

	g->setExclusive( true );
	g->setRadioButtonExclusive( true );
	g->setHidden( true );


	connect( g, SIGNAL( clicked( int ) ), this, SLOT( draw() ) );
	drawingThread = new DrawingThread( this );
	drawingThread->start();
	timerElapsed();

#ifndef NDEBUG	
// this is only for debugging, to easily find out the id of some particle
	pixmapLabel1->installEventFilter(new MouseEventFilter(this));
#endif
};


void Form1::draw() {
	drawingThread->run();
};

class SimulationThread: public QThread {
	private:
		Form1 *form;
		Simulation *sim;
		double sleepTime, old_st;
		QTime timer;
		int missed_count;  //number of consecutive missed frames
	public:
		SimulationThread( Form1 *form0, Simulation *sim0 ) : form( form0 ), sim( sim0 ),
		sleepTime( 0 ), old_st( 0 ), missed_count( 0 ) {}
		;
		double getSimSpeed() { // how much should the simulation have advanced per 1 second
			return double( form->spinBox2->value() ) / 100.0;
		};
		void run() {
			sleepTime = 0;
			timer.restart();
			while ( form->checkBox1->isChecked() ) {
				//			qDebug("SimulationThread running");
				draw();
				int fps = form->spinBox1->value();
				double speed = getSimSpeed();
				//qDebug("fps=%d,slowdown=%f",fps,slowdown);
				//qDebug("advancing sim by %f",1000.0*slowdown/fps);

				sim->advanceBy( speed / fps );
			};
			draw();
			qDebug( "SimulationThread exiting" );

		}
		void draw() {
			double k;

			//			qDebug("minimal sleep");
			//QThread::msleep(20);

			if ( sim == 0 )
				return ;
			//			qDebug("beggining draw");
			k = getSimSpeed();

			SimulationInfo sim0( sim );

			int t = timer.elapsed();
			timer.restart();
			//qDebug("Simulation time(old): %lf",old_st);/**/
			double st = sim0.getTime() - old_st;
			// simulation advanced by st (in sim time)
			old_st = sim0.getTime();
			//qDebug("Simulation time: %lf",old_st);
			//qDebug( "Simulation advanced by %lf", st );
			//qDebug( "elapsed real time: %d", t );
			st /= k;
			st *= 1000;
			// now st contains how much should the sim advance have taken in real time
			// factor 1000 is for conversion to miliseconds
			st -= t;
			// time t has elapsed. Now we want to wait st ms

			//qDebug( "sleepTime: %f (old)", sleepTime );
			//qDebug( "st: %f", st );
			sleepTime += st;
			qDebug( "sleepTime: %f", sleepTime );
			if ( sleepTime <= 0 && missed_count<5) {
				//k*=2;
				//qDebug( "too slow" );
				if ( ++missed_count > 5 ) {
					missed_count = 0;
					sleepTime = 0;
					//qDebug( "reseting sleepTime" );
				};

				return ;
			} else
				missed_count = 0;
			if (sleepTime<0) sleepTime=0;
			qDebug( "Drawing thread waiting for %d", ( int ) sleepTime );
			while ( form->checkBox1->isChecked() ) {
				if ( sleepTime < 100 ) {
					QThread::msleep( ( int ) ( sleepTime ) );
					sleepTime = 0;
					break;
				}

				QThread::msleep( 100 );
				sleepTime -= 100;
			}
			qDebug( "simTime=%f",sim0.getTime() );
			int n = sim->getParticleCount();
			double maxx = sim0.getMaxX();
			double maxy = sim0.getMaxY();

			QButton *b = g->selected();
			QString s = b->text();

			qApp->lock ()
			;
			QPixmap *pixmap = pixmapLabel1->pixmap();

			if ( s == "Particles" ) {
				pixmap->resize( maxx, maxy );
				pixmap->fill();
				QPainter paint( pixmap );
				QtCanvas canvas( paint );
				paint.setClipping( false );
				paint.setPen( Qt::blue );
				for ( int i = 0;i != n;i++ ) {
					ParticleInfo p = sim0.getParticle( i );
					p.draw( canvas );
					double r = p.radius();
					//paint.drawText(p.x()-r,p.y()-r,2*r,2*r,Qt::AlignHCenter |Qt::AlignVCenter,QString::number(p.id()));
				};
				n = sim->getWallCount();
				for ( int i = 0;i < n;i++ ) {
					WallInfo w = sim0.getWall( i );
					w.draw( canvas );
				};
				n = sim->getCompositeCount();
				for ( int i = 0;i < n;i++ ) {
					sim0.getComposite( i ).draw( canvas );
				};
				
				paint.setPen(Qt::black);
				paint.drawText(20,20,QString::number(sim0.getTime()));
				/*				static int counter=0;
							paint.drawText(100,100,QString::number(counter));
							qDebug("counted=%d\n",counter);
							counter++;  */
			} else if ( s == "Density" || s=="Temperature") {

				pixmap->resize( maxx, maxy );
				pixmap->fill();
				QPainter paint( pixmap );

				paint.setClipping( false );

				paint.setPen( Qt::black );
				paint.setBackgroundColor( Qt::white );
				//	paint.fillRect(paint.window(),Qt::white);

				int cx = ( maxx - 1 ) / bucketWidth + 1;
				int cy = ( maxy - 1 ) / bucketHeight + 1;

				//		int *count=new int[cx*cy];
				std::vector<int> count;
				std::vector<double> val;
				count.resize(cx*cy);
				val.resize( cx * cy );

				double maxd;
				for ( int i = 0;i < cx*cy;i++ ) {
					val[ i ] = 0;
					count[ i ]=0;
				};
				if (s=="Density") {
					for ( int i = 0;i < n;i++ ) {
						ParticleInfo p = sim0.getParticle( i );
						int x1 = floor( p.x() / bucketWidth );
						int y1 = floor( p.y() / bucketHeight );
						count[ x1 * cy + y1 ] ++;
					};
					double maxd = double( 4 * n ) / ( cx * cy );
					for ( int i=0; i< cx*cy; i++)
						val[i]=count[i]/maxd;
					
				} else if (s=="Temperature") {
					for ( int i = 0;i < n;i++ ) {
						ParticleInfo p = sim0.getParticle( i );
						double v=hypot(p.vx(),p.vy());
						int x1 = floor( p.x() / bucketWidth );
						int y1 = floor( p.y() / bucketHeight );

						val[ x1 * cy + y1 ] +=v;
						count[ x1 * cy + y1 ] ++;
					};
					for ( int i=0; i< cx*cy; i++) {
						if (count[i]>0)
							val[i]/=4*count[i];
						else
							val[i]=0;
					};

				};
				for ( int x1 = 0;x1 < cx; x1++ ) {
					for ( int y1 = 0;y1 < cy; y1++ ) {
						double c = val[ x1 * cy + y1 ];
						if ( c > 1 )
							c = 1;
						int r = round( c * 255 );
						int g = 0;
						int b = round( ( 1 - c ) * 255 );
						paint.fillRect( x1 * bucketWidth, y1 * bucketHeight, bucketWidth, bucketHeight, QColor( r, g, b ) );
					}
				}
				//		delete []count;
			} else if ( s == "Speed" ) {
				int maxx = scrollView->visibleWidth();
				int maxy = scrollView->visibleHeight();
				qDebug("size: %d,%d",maxx,maxy);
				pixmap->resize( maxx, maxy );
				pixmap->fill();
				QPainter paint( pixmap );

				paint.setPen( Qt::black );
				paint.setBrush( Qt::yellow );
				const int wr = 15;  // width of a rectangle that represents the bucket
				const int c = pixmapLabel1->width() / wr;  //number of buckets

				int *count = new int[ c ];
				for ( int i = 0;i < c;i++ )
					count[ i ] = 0;
				double v_avg = 0;
				for ( int i = 0;i < n;i++ ) {
					ParticleInfo p = sim0.getParticle( i );
					double v=hypot(p.vx(),p.vy());
					//double v = atan2( p.vy(), p.vx() ) + M_PI;
					int b = floor( v / speedInterval );
					if ( b < c )
						count[ b ] ++;
					v_avg += v * v;
				}
				v_avg /= n;
				v_avg = sqrt( v_avg );
				double m = 1;
				double kT = 0.5 * m * v_avg * v_avg;
				int maxcount = 0;
				for ( int i = 0;i < c;i++ )
					if ( count[ i ] > maxcount )
						maxcount = count[ i ];
				for ( int i = 0;i < c;i++ ) {
					int y0 = pixmapLabel1->height() - 10;
					paint.drawRect( i * wr, pixmapLabel1->height() - 1,
					                wr, int( -round( y0 * double( count[ i ] ) / maxcount ) - 1 ) );
				}
				int old_y = pixmapLabel1->height();

				paint.setPen( Qt::blue );
				for ( int x = 0;x < pixmapLabel1->width();x++ ) {
					double v = double( x ) / wr * speedInterval;
					int y0 = pixmapLabel1->height() - 10;
					double v_max = sqrt( 2 * kT / m );
					double zoom = y0 / ( 4 * M_PI * pow( m / ( 2 * M_PI * kT ), 1.5 ) * v_max * v_max * exp( -m * v_max * v_max / ( 2 * kT ) ) );
					int y = pixmapLabel1->height() - int( zoom * 4 * M_PI * pow( m / ( 2 * M_PI * kT ), 1.5 ) * v * v * exp( -m * v * v / ( 2 * kT ) ) );
//					paint.drawLine( x - 1, old_y, x, y );
					old_y = y;

					v = sqrt( 2 * kT / m );
					y = pixmapLabel1->height() - int( zoom * 4 * M_PI * pow( m / ( 2 * M_PI * kT ), 1.5 ) * v * v * exp( -m * v * v / ( 2 * kT ) ) );
//					paint.drawLine( 0, y, 600, y );


				};
				//	paint.drawRect(int(sqrt(2*kT/m)/speedInterval*wr),0,2,pixmapLabel1->height());
				//	paint.setPen(Qt::red);
//				paint.drawRect( int( v_avg / speedInterval * wr ), 0, 2, 1000 );
				paint.setPen( Qt::blue );
				delete [] count;
			}
			//			qApp->lock();
			pixmapLabel1->resize( pixmap->size() );
			pixmapLabel1->repaint( false );
			//unsigned int wait_time=form->spinBox2->value()*(1000/form->spinBox1->value())/100;
			qApp->unlock();

			//QThread::msleep(wait_time);
		};

};

void Form1::run_sim() {
	if ( !checkBox1->isChecked() ) {
		if ( sim != 0 ) {
			sim->stop();
			draw();
		};
		return ;
	}
	try {
		QTimer::singleShot( 1000 / spinBox1->value(), this, SLOT( timerElapsed() ) );
		if ( sim == 0 )
			sim = init_sim( scriptName.ascii() );
		if ( sim == 0 )
			return ;
		/*		{
					SimulationInfo sim0(sim);
					double maxx=sim0.getMaxX();
					double maxy=sim0.getMaxY();
					pixmapLabel1->pixmap()->resize(int(maxx),int(maxy));
					pixmapLabel1->resize(int(maxx),int(maxy));
				}*/

		static SimulationThread st( this, sim );
		if ( !st.running() )
			st.start();
		setInterval();
	} catch ( std::runtime_error e ) {
		checkBox1->setChecked( false );
		QMessageBox mb( "Error", e.what(), QMessageBox::Critical, QMessageBox::Ok,
		                QMessageBox::NoButton, QMessageBox::NoButton, this, 0, true );
		mb.exec();
	};
};



void Form1::pushButton1_pressed() {
	QPainter paint( pixmapLabel1->pixmap() );
	int x1 = QInputDialog::getInteger( "", "" );
	int y1 = QInputDialog::getInteger( "", "" );
	int x2 = QInputDialog::getInteger( "", "" );
	int y2 = QInputDialog::getInteger( "", "" );
	paint.fillRect( x1, y1, x2, y2, Qt::yellow );
	pixmapLabel1->repaint( false );
}


void Form1::chooseInitScript() {
	scriptName = QFileDialog::getOpenFileName( ".", "*.py", this );
	delete sim;
	sim = 0;
}


void Form1::displaySettings() {
	DisplaySettingsDialog * d = new DisplaySettingsDialog( this, "Display Settings" );
	d->spinBox2->setValue( bucketWidth );
	d->spinBox3->setValue( bucketHeight );
	d->spinBox5->setValue( int( speedInterval * 1000 ) );

	if ( d->exec() == QDialog::Accepted ) {
		bucketWidth = d->spinBox2->value();
		bucketHeight = d->spinBox3->value();
		speedInterval = double( d->spinBox5->value() ) / 1000;
	};
	delete d;

}

void Form1::draw_sync() {}
;

void Form1::setInterval() {}
;

void Form1::destroy() {
	checkBox1->setChecked( false );
	if ( sim != 0 )
		sim->stop();
	
	delete sim;
	sim = 0;
};

void Form1::beforeQuit() {
	checkBox1->setChecked( false );
};

void Form1::timerElapsed() {
	//	qDebug("drawing");
	//	draw();
	//	qDebug("processing events");
	//	qDebug(qApp->hasPendingEvents()?"there are events":"no events");
	for ( int i = 0;i < 10;i++ )
		qApp->processEvents( 1000 );
	//	qDebug("scheduling timer");
	if ( checkBox1->isChecked() )
		QTimer::singleShot( 1000 / spinBox1->value(), this, SLOT( timerElapsed() ) );
	//	qDebug("end");
};