#include "precomp.h"
#include "gas/simulation.h"
#include "python_export.h"
#include "init_sim.h"
#include "gas/common.h"
#include <cstdio>

using namespace std;
using namespace boost::python;


Simulation* init_sim(const char* scriptname) {
	static bool py_initialized=false;
	if (!py_initialized) {
		if (PyImport_AppendInittab("gas",initgas)==-1) 
			throw std::runtime_error("Couldn't initialize gas module") ;
		Py_Initialize();
		py_initialized=true;
	};
	
	object main_module((
	    handle<>(borrowed(PyImport_AddModule("__main__")))));
	object main_namespace = main_module.attr("__dict__");
	
	Simulation *result=0;
	try {
		//first parse the script file
        FILE* fp=fopen(scriptname,"r");

		handle<> ignored((PyRun_File(
                          fp
                        , scriptname
						, Py_file_input
					    , main_namespace.ptr()
						, main_namespace.ptr())
					));
        fclose(fp);
		cout << "Script file parsed" << endl;
		// then insert variable "infinity"
		main_namespace["infinity"]=infinity;
		// then acquire the init_sim object
		// which should be a function taking no arguments
		// and returning Simulation
		object f=main_namespace["init_sim"];
		object r1=f();
		auto_ptr<Simulation> r2=extract<auto_ptr<Simulation> >(r1);
		result=r2.release();
		cout << "Script ended" << endl;
	} catch (error_already_set) {
		PyObject *a,*b,*c;
		try {
			PyErr_Fetch(&a,&b,&c);
            str omsg;  //error message
            object otb;  //traceback object
			handle<> ha(a);
            str oexception(ha);
            if (b) { 
    			handle<> hb(b);
                str t(hb);
                omsg=t;
            };
            if (c) {
    			handle<> hc(c);
                object t(hc);
                otb=t;
			}
			ostringstream ss;
			string exception=extract<string>(oexception)();
			string msg=extract<string>(omsg)();
            int line=-1;
            try {
    			line=extract<int>(otb.attr("tb_frame").attr("f_lineno"))(); /**/
            } catch (error_already_set) {
				PyErr_Print();
            };
/*			string exception="exception";
			string msg="msg";
			int line=6;/**/
			ss << "Python error "<<exception<<endl << "at line ";
            if (line!=-1) 
				ss <<line;
            else 
				ss << "???";
            ss <<" of script file with message "<< endl<<msg;
			string s=ss.str();
			cout << s << endl;
			throw runtime_error(ss.str());
		} catch (error_already_set) {
			PyErr_Print();
		}
		
		
	};
//	cout << "Before PyFinalize()"<<endl;
//	Py_Finalize();
	return result;
};
